﻿#include "PianoRollEditTool_DragView.h"

#include "SlateApplication.h"
#include "UI/Widget/PianoRoll/PianoRollPanel/SPianoRollPanel.h"

void FPianoRollEditTool_DragView::OnActive(FPianoRollEdit& Edit)
{
	FPianoRollEditTool::OnActive(Edit);
	const TSharedPtr<SPianoRollPanel> Panel = Edit.PianoRollPanel.Pin();
	OriginalRange = Panel->TimeRange;
	OriginalMousePos = FSlateApplication::Get().GetCursorPos();
}

void FPianoRollEditTool_DragView::OnMouseButtonDown(FPianoRollEdit& Edit, const FGeometry& MyGeometry,
                                                      const FPointerEvent& MouseEvent)
{
	OriginalMousePos = MouseEvent.GetScreenSpacePosition();
	const TSharedPtr<SPianoRollPanel> Panel = Edit.PianoRollPanel.Pin();
	OriginalRange = Panel->TimeRange;
}

void FPianoRollEditTool_DragView::OnMouseMove(FPianoRollEdit& Edit, const FGeometry& MyGeometry,
                                                const FPointerEvent& MouseEvent)
{
	if (!MouseEvent.IsMouseButtonDown(EKeys::RightMouseButton))
		return;
	
	const FVector2f MouseCurrentPos = MouseEvent.GetScreenSpacePosition();
	TSharedPtr<SPianoRollPanel> Panel = Edit.PianoRollPanel.Pin();
	const FVector2f Delta = MouseCurrentPos - OriginalMousePos;
	const double TickDelta = Delta.X / Panel->FrameToPixel / MyGeometry.Scale;

	const AudioFrame Lower = FMath::Max<AudioFrame>(0, OriginalRange.GetLowerBoundValue() - TickDelta);
	const AudioFrame Upper = FMath::Max<AudioFrame>(OriginalRange.Size<double>(), OriginalRange.GetUpperBoundValue() - TickDelta);
	Panel->OnTargetRangeChanged.ExecuteIfBound(TRange<AudioFrame>(Lower, Upper));
	Panel->OnYRequestUpdate.ExecuteIfBound(-MouseEvent.GetCursorDelta().Y / MyGeometry.Scale);
}

void FPianoRollEditTool_DragView::OnBeginDrag(FPianoRollEdit& Edit, FVector2f InMouseStartPos)
{
	OriginalMousePos = FSlateApplication::Get().GetCursorPos();
	const TSharedPtr<SPianoRollPanel> Panel = Edit.PianoRollPanel.Pin();
	OriginalRange = Panel->TimeRange;
}
