﻿#include "PianoRollEditTool_Select.h"

#include "SlateApplication.h"
#include "PluginHost/PluginHost.h"
#include "UI/Widget/PianoRoll/PianoRollPanel/SPianoRollPanel.h"

void FPianoRollEditTool_Select::Clear(FPianoRollEdit& Edit)
{
	FPianoRollEditTool::Clear(Edit);
	RectSelecting = false;
}

void FPianoRollEditTool_Select::OnMouseButtonDown(FPianoRollEdit& Edit, const FGeometry& MyGeometry,
                                                    const FPointerEvent& MouseEvent)
{
	if (MouseEvent.GetEffectingButton() == EKeys::LeftMouseButton)
		RectSelecting = true;
}

void FPianoRollEditTool_Select::OnMouseButtonUp(FPianoRollEdit& Edit, const FGeometry& MyGeometry,
	const FPointerEvent& MouseEvent)
{
	if (MouseEvent.GetEffectingButton() == EKeys::LeftMouseButton)
		RectSelecting = false;
}

void FPianoRollEditTool_Select::OnMouseMove(FPianoRollEdit& Edit, const FGeometry& MyGeometry,
	const FPointerEvent& MouseEvent)
{
	MouseCurrentPos = MyGeometry.AbsoluteToLocal(MouseEvent.GetScreenSpacePosition());
}

void FPianoRollEditTool_Select::OnEndDrag(FPianoRollEdit& Edit, FVector2f InMouseEndPos)
{
	Edit.Selector.UnSelect();
	
	const TSharedPtr<SPianoRollPanel> Panel = Edit.PianoRollPanel.Pin();
	const FVector2f& MouseDownPos = Edit.GetMouseDownPos();

	TArray<FMidiMessageSequence::MidiEventHolder*> InRectMidiMessages;
	const int MouseDownNote = FMath::Abs(Panel->HeightToNoteNumber(MouseDownPos.Y));
	const int MouseUpNote = FMath::Abs(Panel->HeightToNoteNumber(InMouseEndPos.Y));
		
	const TRange<int> NoteRange(MouseUpNote, MouseDownNote);
	const TRange<AudioFrame> MouseMidiTickRange(Panel->PixelToFrame(MouseDownPos.X), Panel->PixelToFrame(InMouseEndPos.X));

	FMidiMessageSequence* MidiEventHolders = Edit.GetMidiMessageSequence();
	for (FMidiMessageSequence::MidiEventHolder* MidiMessage : *MidiEventHolders)
	{
		if (!MidiMessage->message.isNoteOn())
			continue;
		
		// 如果MidiMessage的时间戳大于选择框的结束时间，那么就不需要再遍历了
		if (MidiMessage->message.getTimeStamp() > MouseMidiTickRange.GetUpperBoundValue())
			break;
		
		const double NoteOnTime = MidiMessage->message.getTimeStamp();
		const double Duration = MidiMessage->getDuration();
		
		if (!MouseMidiTickRange.Contains(NoteOnTime) && !MouseMidiTickRange.Contains(NoteOnTime + Duration))
			continue;
		
		const int NoteNumber = MidiMessage->message.getNoteNumber();
		if (!NoteRange.Contains(NoteNumber))
			continue;
		
		InRectMidiMessages.Add(MidiMessage);
	}

	Edit.Selector.SelectMidiMessage(InRectMidiMessages);
	PlaySelected(Edit);
}

int32 FPianoRollEditTool_Select::OnPaint(const FPianoRollEdit& Edit, const FPaintArgs& Args, const FGeometry& AllottedGeometry,
                                         const FSlateRect& MyCullingRect, FSlateWindowElementList& OutDrawElements, int32 LayerId,
                                         const FWidgetStyle& InWidgetStyle, bool bParentEnabled) const
{
	if (!RectSelecting)
		return LayerId;
	// 绘制选择框
	const FVector2f& MouseDownPos = Edit.GetMouseDownPos();
	const FVector2f RectStart = FVector2f(MouseDownPos.X, MouseDownPos.Y);
	const FVector2f RectEnd = FVector2f(MouseCurrentPos.X, MouseCurrentPos.Y);
	
	// Draw Line
	TArray<FVector2f> Points;
	Points.Add(RectStart);
	Points.Add(FVector2f(RectEnd.X, RectStart.Y));
	Points.Add(RectEnd);
	Points.Add(FVector2f(RectStart.X, RectEnd.Y));
	Points.Add(RectStart);
	FSlateDrawElement::MakeLines(
		OutDrawElements,
		++LayerId,
		AllottedGeometry.ToPaintGeometry(),
		Points,
		ESlateDrawEffect::None,
		FLinearColor::Red,
		true,
		1
	);
	
	return LayerId;
}
