﻿#pragma once
#include "CoreMinimal.h"
#include "Events.h"
#include "Geometry.h"
#include "PaintArgs.h"
#include "PianoRollSelector.h"
#include "Reply.h"
#include "WidgetStyle.h"
#include "Midi/MidiMessageSequence.h"
#include "Midi/MidiType.h"
#include "Midi/Time/TimePos.h"
class FPluginHost;
class FPianoRollEditTool;
class SPianoRollPanel;

enum class EPianoRollToolType
{
	Select,
	Write,
	DragView,
	Count
};

struct FNoteSelectData
{
	FMidiMessageSequence::MidiEventHolder* MidiMessage = nullptr;
	double MidiTimestampOffset = 0;
	operator bool() const
	{
		return MidiMessage != nullptr;
	}
};

class FPianoRollEdit
{
public:
	FPianoRollEdit(TSharedRef<SPianoRollPanel> InPianoRollPanel);
	~FPianoRollEdit();

	FNoteSelectData GetMidiMessageByPos(FVector2D Pos) const;
	FNoteSelectData GetMidiMessageUnderMouse() const;

	FReply OnMouseButtonDown(const FGeometry& MyGeometry, const FPointerEvent& MouseEvent);
	FReply OnMouseButtonUp(const FGeometry& MyGeometry, const FPointerEvent& MouseEvent);
	FReply OnMouseMove(const FGeometry& MyGeometry, const FPointerEvent& MouseEvent);
	FReply OnKeyDown(const FGeometry& MyGeometry, const FKeyEvent& KeyEvent);
	FReply OnKeyUp(const FGeometry& MyGeometry, const FKeyEvent& KeyEvent);
	int32 OnPaint(const FPaintArgs& Args, const FGeometry& AllottedGeometry, const FSlateRect& MyCullingRect, FSlateWindowElementList& OutDrawElements, int32 LayerId, const FWidgetStyle& InWidgetStyle, bool bParentEnabled) const;

	int32 GetNoteNumberByPos(float Height) const;
	int32 GetNoteNumberUnderMouse() const;
	AudioFrame GetAudioFrameByPos(float Width) const;
	AudioFrame GetAudioFrameUnderMouse() const;
	
	bool IsNoteSelected(FMidiMessageSequence::MidiEventHolder* MidiMessage) const { return Selector.IsSelected(MidiMessage); }
	bool IsMultiSelected() const { return Selector.Num() > 1; }
	FVector2f GetMouseDownPos() const { return MouseDownPos; }
	void SetModified(bool InHasModified) { HasModified = InHasModified; }
	bool GetModified() const { return HasModified; }
	bool IsMouseDragging() const { return OnMouseDrag; }
	AudioFrame GetSnapFrame() const;
	FMidiMessageSequence* GetMidiMessageSequence() const;
	FPluginHost* GetPluginHost() const;
	
	void SetTempTool(EPianoRollToolType InToolType);
	void ClearTempTool();

	FPianoRollSelector Selector;
	TWeakPtr<SPianoRollPanel> PianoRollPanel;
protected:
	bool OnMouseDrag = false;
private:
	void UpdateModified();
	FPianoRollEditTool* TryEnableTempTool();
	
	FVector2f MouseDownPos;
	bool HasModified = false;

	TArray<FKey> PressedKeys;

	FPianoRollEditTool* GetCurrentTool() const { return TempTool ? TempTool : CurrentEditTool; }
	FPianoRollEditTool* TempTool = nullptr;
	FPianoRollEditTool* CurrentEditTool = nullptr;
	TMap<EPianoRollToolType, FPianoRollEditTool*> EditToolMap;
	TMap<EPianoRollToolType, TArray<FKey>> ToolKeyMap;
};
