﻿#include "PianoRollSelector.h"

void FPianoRollSelector::SelectMidiMessage(TArray<FMidiMessageSequence::MidiEventHolder*> InMidiMessages)
{
	UnSelect();
	for (auto MidiMessage : InMidiMessages)
	{
		FData& D = SelectedMidiMessages.AddDefaulted_GetRef();
		D.MidiMessage = MidiMessage;
		D.OriginalPitch = MidiMessage->message.getNoteNumber();
		D.OriginalMidiTimestamp = MidiMessage->message.getTimeStamp();
		
		OnMidiMessageSelected.Broadcast(MidiMessage);
	}
}

void FPianoRollSelector::AddMidiMessage(FMidiMessageSequence::MidiEventHolder* MidiMessage)
{
	if (SelectedMidiMessages.Contains(MidiMessage))
		return;
	
	FData& D = SelectedMidiMessages.AddDefaulted_GetRef();
	D.MidiMessage = MidiMessage;
	D.OriginalPitch = MidiMessage->message.getNoteNumber();
	D.OriginalMidiTimestamp = MidiMessage->message.getTimeStamp();
	
	OnMidiMessageSelected.Broadcast(MidiMessage);
}

void FPianoRollSelector::UnSelect()
{
	for (const auto& D : SelectedMidiMessages)
	{
		OnMidiMessageUnSelected.Broadcast(D.MidiMessage);
	}
	SelectedMidiMessages.Reset();
}

void FPianoRollSelector::UnSelect(FMidiMessageSequence::MidiEventHolder* MidiMessage)
{
	if (SelectedMidiMessages.RemoveAll([&MidiMessage](const FData& Element) { return Element == MidiMessage; }) != 0)
	{
		OnMidiMessageUnSelected.Broadcast(MidiMessage);
	}
}

bool FPianoRollSelector::SetPitch(int Pitch)
{
	const int PitchDelta = Pitch - BasePitch;
	for (const FData& D : SelectedMidiMessages)
	{
		const int NewNoteNumber = D.OriginalPitch + PitchDelta;
		D.MidiMessage->message.setNoteNumber(NewNoteNumber);
		if (D.MidiMessage->noteOffObject)
			D.MidiMessage->noteOffObject->message.setNoteNumber(NewNoteNumber);
	}
	
	if (LastPitch != Pitch)
	{
		LastPitch = Pitch;
		return true;
	}
	return false;
}

bool FPianoRollSelector::SetMidiTimestamp(MidiTick InMidiTick)
{
	MidiTick TimestampOffset = BaseMidiTimestamp - InMidiTick + MidiTimestampDelta; // 点击位置的MidiTick
	TimestampOffset = FMath::GridSnap<MidiTick>(TimestampOffset, SnapMidiTimestamp);

	for (const FData& D : SelectedMidiMessages)
	{
		const double NewTimestamp = FMath::Max(0, D.OriginalMidiTimestamp - TimestampOffset);
		
		const double Duration = D.MidiMessage->getDuration();
		D.MidiMessage->message.setTimeStamp(NewTimestamp);
		if (D.MidiMessage->noteOffObject)
			D.MidiMessage->noteOffObject->message.setTimeStamp(NewTimestamp + Duration);
	}

	if (LastMidiTimestamp != TimestampOffset)
	{
		LastMidiTimestamp = TimestampOffset;
		return true;
	}
	return false;
}

void FPianoRollSelector::ResetAnchorPoint()
{
	for (FData& D : SelectedMidiMessages)
	{
		D.OriginalPitch = D.MidiMessage->message.getNoteNumber();
		D.OriginalMidiTimestamp = D.MidiMessage->message.getTimeStamp();
	}
}

bool FPianoRollSelector::IsSelected(FMidiMessageSequence::MidiEventHolder* MidiMessage) const
{
	return SelectedMidiMessages.Contains(MidiMessage);
}

TArray<FMidiMessageSequence::MidiEventHolder*> FPianoRollSelector::GetSelectedMidiMessages() const
{
	TArray<FMidiMessageSequence::MidiEventHolder*> OutSelectedMidiMessages;
	for (const FData& D : SelectedMidiMessages)
	{
		OutSelectedMidiMessages.Add(D.MidiMessage);
	}
	return OutSelectedMidiMessages;
}
