﻿#pragma once
#include "CoreMinimal.h"
#include "Midi/MidiMessageSequence.h"
#include "Midi/Time/TimePos.h"

DECLARE_MULTICAST_DELEGATE_OneParam(FMidiMessageDelegate, FMidiMessageSequence::MidiEventHolder*)

class FPianoRollSelector
{
	struct FData
	{
		FMidiMessageSequence::MidiEventHolder* MidiMessage;
		int OriginalPitch;
		double OriginalMidiTimestamp;

		bool operator==(FMidiMessageSequence::MidiEventHolder* In) const
		{
			return MidiMessage == In;
		}
	};
public:
	void SelectMidiMessage(TArray<FMidiMessageSequence::MidiEventHolder*> InMidiMessages);
	void AddMidiMessage(FMidiMessageSequence::MidiEventHolder* MidiMessage);
	void UnSelect();
	void UnSelect(FMidiMessageSequence::MidiEventHolder* MidiMessage);
	
	void BeginDrag(int InBasePitch, MidiTick InMidiTimestampDelta, MidiTick InSnapMidiTimestamp, MidiTick InBaseMidiTimestamp)
	{
		BasePitch = InBasePitch;
		MidiTimestampDelta = InMidiTimestampDelta;
		SnapMidiTimestamp = InSnapMidiTimestamp;
		BaseMidiTimestamp = InBaseMidiTimestamp;
		
		LastPitch = InBasePitch;
		LastMidiTimestamp.Ticks = 0;
	}
	void ResetAnchorPoint();

	bool SetPitch(int Pitch);
	bool SetMidiTimestamp(MidiTick InMidiTick);
	bool IsSelected(FMidiMessageSequence::MidiEventHolder* MidiMessage) const;
	
	TArray<FMidiMessageSequence::MidiEventHolder*> GetSelectedMidiMessages() const;
	int32 Num() const { return SelectedMidiMessages.Num(); }

	FMidiMessageDelegate OnMidiMessageSelected;
	FMidiMessageDelegate OnMidiMessageUnSelected;
private:
	TArray<FData> SelectedMidiMessages;
	int32 BasePitch = 0;
	MidiTick MidiTimestampDelta;
	MidiTick BaseMidiTimestamp;
	MidiTick SnapMidiTimestamp;
	
	int32 LastPitch = 0;
	MidiTick LastMidiTimestamp;
};
