﻿// Fill out your copyright notice in the Description page of Project Settings.

#pragma once

#include "CoreMinimal.h"
#include "PianoRollEdit.h"
#include "SCompoundWidget.h"
#include "SLeafWidget.h"
#include "Midi/MidiMessageSequence.h"

class FMidiMessage;
class FMidiMessageSequence;

DECLARE_DELEGATE_OneParam(FPianoRollPanelTargetRangeChanged, TRange<AudioFrame>)
DECLARE_DELEGATE_OneParam(FPianoRollPanelYRequestUpdate, float /*Delta*/)
DECLARE_DELEGATE_OneParam(FPianoRollPanelRequestHScroll, bool /*Delta*/)

/**
 * 
 */
class ARONA_API SPianoRollPanel : public SLeafWidget
{
	friend class FPianoRollEdit;
public:
	SLATE_BEGIN_ARGS(SPianoRollPanel):
		_SpacingLineColor(FLinearColor::Gray),
		_NoteColor(FLinearColor::White),
		_NoteSelectedColor(FLinearColor::Blue),
		_NoteTextColor(FLinearColor::Black)
		{
		}

	SLATE_EVENT(FPianoRollPanelTargetRangeChanged, OnTargetRangeChanged)
	SLATE_EVENT(FPianoRollPanelYRequestUpdate, OnYRequestUpdate)
	SLATE_EVENT(FPianoRollPanelRequestHScroll, OnRequestHScroll)

	SLATE_ARGUMENT(const FSlateBrush*, NoteImage)
	
	SLATE_ARGUMENT(FLinearColor, SpacingLineColor)
	SLATE_ARGUMENT(FLinearColor, NoteColor)
	SLATE_ARGUMENT(FLinearColor, NoteSelectedColor)
	SLATE_ARGUMENT(FLinearColor, NoteTextColor)
	SLATE_ARGUMENT(FSlateFontInfo, FontInfo)

	SLATE_ARGUMENT(float, SpacingLineThickness)
	SLATE_ARGUMENT(float, NoteHeight)

	
	SLATE_END_ARGS()

	/** Constructs this widget with InArgs */
	void Construct(const FArguments& InArgs, FMidiMessageSequence* InMidiMessageSequence);

	virtual int32 OnPaint(const FPaintArgs& Args, const FGeometry& AllottedGeometry, const FSlateRect& MyCullingRect, FSlateWindowElementList& OutDrawElements, int32 LayerId, const FWidgetStyle& InWidgetStyle, bool bParentEnabled) const override;
	virtual FVector2D ComputeDesiredSize(float LayoutScaleMultiplier) const override { return FVector2D(SequenceWidth, NoteHeight * 128); }

	virtual FReply OnMouseButtonDown(const FGeometry& MyGeometry, const FPointerEvent& MouseEvent) override;
	virtual FReply OnMouseButtonUp(const FGeometry& MyGeometry, const FPointerEvent& MouseEvent) override;
	virtual FReply OnMouseMove(const FGeometry& MyGeometry, const FPointerEvent& MouseEvent) override;
	virtual FReply OnKeyDown(const FGeometry& MyGeometry, const FKeyEvent& InKeyEvent) override;
	virtual FReply OnKeyUp(const FGeometry& MyGeometry, const FKeyEvent& InKeyEvent) override;
	virtual bool SupportsKeyboardFocus() const override { return true; }
	
	int HeightToNoteNumber(float Height) const { return FMidiMessage::MaxNoteNumber - Height / NoteHeight + 1; }
	// double PixelToMidiTick(float Pixel) const { return Pixel / FrameToPixel + TimeRange.GetLowerBoundValue(); }
	AudioFrame PixelToFrame(float Pixel) const { return Pixel / FrameToPixel + TimeRange.GetLowerBoundValue(); }
	
	FMidiMessageSequence* GetMidiMessageSequence() const { return MidiMessageSequence; }
	
	TRange<AudioFrame> TimeRange;
	
	FPianoRollPanelTargetRangeChanged OnTargetRangeChanged;
	FPianoRollPanelYRequestUpdate OnYRequestUpdate;
	FPianoRollPanelRequestHScroll OnRequestHScroll;
	
	FLinearColor SpacingLineColor;
	FLinearColor NoteColor;
	FLinearColor NoteSelectedColor;
	FLinearColor NoteTextColor;
	FSlateFontInfo FontInfo;
	TSharedPtr<FPianoRollEdit> PianoRollEdit;
	float SpacingLineThickness = 1.f;
	float NoteHeight = 20.f;
	float FrameToPixel = 0.2f;
private:
	float SequenceWidth = 100.f;
	const FSlateBrush* BackgroundImage = nullptr;
	const FSlateBrush* NoteImage = nullptr;
	FMidiMessageSequence* MidiMessageSequence = nullptr;
};
