﻿#include "PlayListEditTool_Select.h"

#include "UI/Widget/PlayList/PlayListPanel/SPlayListPanel.h"

void FPlayListEditTool_Select::Clear(FPlayListEdit& Edit)
{
	FPlayListEditTool::Clear(Edit);
	RectSelecting = false;
}

void FPlayListEditTool_Select::OnMouseMove(FPlayListEdit& Edit, const FGeometry& MyGeometry,
                                           const FPointerEvent& MouseEvent)
{
	MouseCurrentPos = MyGeometry.AbsoluteToLocal(MouseEvent.GetScreenSpacePosition());
}

void FPlayListEditTool_Select::OnBeginDrag(FPlayListEdit& Edit, FVector2f InMouseStartPos)
{
	RectSelecting = true;
}

void FPlayListEditTool_Select::OnEndDrag(FPlayListEdit& Edit, FVector2f InMouseEndPos)
{
	RectSelecting = false;
	
	const FVector2f& MouseDownPos = Edit.GetMouseDownPos();
	const TSharedPtr<SPlayListPanel> Panel = Edit.PlayListPanel.Pin();
	FPlayListSelector& Selector = Edit.Selector;
	
	Selector.UnSelect();
	const int32 BeginTrackIndex = Panel->TrackHeightManager.GetTrackIndexAt(MouseDownPos.Y);
	const int32 EndTrackIndex = Panel->TrackHeightManager.GetTrackIndexAt(InMouseEndPos.Y);

	const TRange<int32> TrackRange = TRange<int32>(BeginTrackIndex, EndTrackIndex);
	const TRange<AudioFrame> MidiTimestampRange = TRange<AudioFrame>(Panel->PixelToFrame(MouseDownPos.X), Panel->PixelToFrame(InMouseEndPos.X));

	FMidiSequencer::Get().ForeachPatternInstance([&](FPatternInstance* Instance) ->bool
	{
		const AudioFrame InstanceBegin = Instance->Pos;
		const AudioFrame InstanceEnd = Instance->Pos + Instance->GetLength();
		if (TrackRange.Contains(Instance->TrackIndex) && MidiTimestampRange.Contains(InstanceBegin) && MidiTimestampRange.Contains(InstanceEnd))
		{
			Selector.AddPatternInstance(Instance);
		}
		return true;
	});
}

int32 FPlayListEditTool_Select::OnPaint(const FPlayListEdit& Edit, const FPaintArgs& Args,
	const FGeometry& AllottedGeometry, const FSlateRect& MyCullingRect, FSlateWindowElementList& OutDrawElements,
	int32 LayerId, const FWidgetStyle& InWidgetStyle, bool bParentEnabled) const
{
	if (!RectSelecting)
		return LayerId;
	
	// 绘制选择框
	const FVector2f& MouseDownPos = Edit.GetMouseDownPos();
	const FVector2f RectStart = FVector2f(MouseDownPos.X, MouseDownPos.Y);
	const FVector2f RectEnd = FVector2f(MouseCurrentPos.X, MouseCurrentPos.Y);
	
	// Draw Line
	TArray<FVector2f> Points;
	Points.Add(RectStart);
	Points.Add(FVector2f(RectEnd.X, RectStart.Y));
	Points.Add(RectEnd);
	Points.Add(FVector2f(RectStart.X, RectEnd.Y));
	Points.Add(RectStart);
	FSlateDrawElement::MakeLines(
		OutDrawElements,
		++LayerId,
		AllottedGeometry.ToPaintGeometry(),
		Points,
		ESlateDrawEffect::None,
		FLinearColor::Red,
		true,
		1
	);
	
	return LayerId;
}
