﻿#include "PlayListEditTool_Write.h"

#include "Pattern/PatternInstance.h"

void FPlayListEditTool_Write::Clear(FPlayListEdit& Edit)
{
	FPlayListEditTool::Clear(Edit);
	TempSelector.UnSelect();
}

void FPlayListEditTool_Write::OnMouseButtonDown(FPlayListEdit& Edit, const FGeometry& MyGeometry,
                                                const FPointerEvent& MouseEvent)
{
	if (MouseEvent.GetEffectingButton() == EKeys::LeftMouseButton)
	{
		LeftMouseButton(Edit);
	}
	
	if (MouseEvent.GetEffectingButton() == EKeys::RightMouseButton)
	{
		RightMouseButton(Edit);
	}
}

void FPlayListEditTool_Write::OnMouseButtonUp(FPlayListEdit& Edit, const FGeometry& MyGeometry,
	const FPointerEvent& MouseEvent)
{
	TempSelector.UnSelect();
}

void FPlayListEditTool_Write::OnMouseMove(FPlayListEdit& Edit, const FGeometry& MyGeometry,
	const FPointerEvent& MouseEvent)
{
	const FGeometry& TickSpaceGeometry = MyGeometry;
	const auto& CursorPos = MouseEvent.GetScreenSpacePosition();
	const auto& LocalPos = TickSpaceGeometry.AbsoluteToLocal(CursorPos);

	if (MouseEvent.IsMouseButtonDown(EKeys::LeftMouseButton))
	{
		const int TrackIndex = Edit.GetTrackIndexByPos(LocalPos.Y);
		const AudioFrame NewFrame = Edit.PixelToFrame(LocalPos.X);
		UpdateTempSelector(Edit, TrackIndex, NewFrame);
		UpdateSelector(Edit, TrackIndex, NewFrame);
	}

	if (MouseEvent.IsMouseButtonDown(EKeys::RightMouseButton))
	{
		RightMouseButton(Edit);
	}
}

void FPlayListEditTool_Write::LeftMouseButton(FPlayListEdit& Edit)
{
	const FPatternInstanceSelectData& PatternInstanceSelectData = Edit.GetPatternUnderCursor();

	FPlayListSelector& Selector = Edit.Selector;
	const AudioFrame SnapMidiTimestamp = Edit.GetSnapFrame();

	if (PatternInstanceSelectData)
	{
		FPatternInstance* PatternInstance = PatternInstanceSelectData.PatternInstance;
		const int32 BaseTrackIndex = PatternInstance->TrackIndex;
		const AudioFrame MidiTimestampOffset = PatternInstanceSelectData.MidiTimestampOffset;
		const AudioFrame BaseMidiTimestamp = PatternInstance->Pos;
		
		FPatternSelector::Get().SelectPatternInstance(PatternInstance);
		
		if (Selector.IsSelected(PatternInstance))
		{
			Selector.BeginDrag(BaseTrackIndex, SnapMidiTimestamp, MidiTimestampOffset, BaseMidiTimestamp);
			Selector.ResetAnchorPoint();
		}
		else
		{
			Selector.UnSelect();
			TempSelector.UnSelect();
			TempSelector.AddPatternInstance(PatternInstance);
			TempSelector.BeginDrag(BaseTrackIndex, SnapMidiTimestamp, MidiTimestampOffset, BaseMidiTimestamp);
			TempSelector.ResetAnchorPoint();
		}
		return;
	}

	Selector.UnSelect();

	FPatternSelector& PatternSelector = FPatternSelector::Get();
	FPattern* SelectedPattern = PatternSelector.GetSelectedPattern();
	if (!SelectedPattern)
		return;

	const int32 TrackIndex = Edit.GetTrackIndexUnderCursor();
	const AudioFrame FrameUnderMouse = Edit.GetFrameUnderCursor();
	const AudioFrame TimestampDelta = FMath::Fmod(FrameUnderMouse, SnapMidiTimestamp);
	const AudioFrame SnapFrame = FrameUnderMouse - TimestampDelta;
	const TRange<AudioFrame> Range = PatternSelector.GetRange();
	const TFuture<FPatternInstance*> Future =
		SelectedPattern->RequestCreateInstance(
			TrackIndex,
			SnapFrame,
			Range.GetLowerBoundValue(),
			Range.GetUpperBoundValue());
	// Future.WaitFor(FTimespan::FromSeconds(1)); // 等待1秒如果还没有创建完成就放弃
	// FPatternInstance* Instance = Future.Get();
	// if (!Instance)
	// 	return;
	// TempSelector.BeginDrag(TrackIndex, SnapMidiTimestamp, SnapMidiTick, MidiTickUnderMouse);
	// TempSelector.AddPatternInstance(Instance);
}

void FPlayListEditTool_Write::RightMouseButton(FPlayListEdit& Edit)
{
	const FPatternInstanceSelectData& PatternInstanceSelectData = Edit.GetPatternUnderCursor();
	if (PatternInstanceSelectData)
	{
		FPatternInstance* PatternInstance = PatternInstanceSelectData.PatternInstance;
		PatternInstance->RequestDestroy();
	}
}

void FPlayListEditTool_Write::UpdateTempSelector(FPlayListEdit& Edit, int32 NewTrackIndex, AudioFrame NewFrame)
{
	if (TempSelector.Num() == 0)
		return;
	TempSelector.SetTrackIndex(NewTrackIndex);
	if (TempSelector.SetFrameTimestamp(NewFrame))
	{
		TempSelector.EndSetting();
	}
}

void FPlayListEditTool_Write::UpdateSelector(FPlayListEdit& Edit, int32 NewTrackIndex, AudioFrame NewFrame)
{
	FPlayListSelector& Selector = Edit.Selector;
	if (Selector.Num() == 0)
		return;
	Selector.SetTrackIndex(NewTrackIndex);
	if (Selector.SetFrameTimestamp(NewFrame))
	{
		Selector.EndSetting();
	}
}
