﻿#pragma once
#include "PaintArgs.h"
#include "PlayListSelector.h"
#include "Reply.h"
#include "WidgetStyle.h"

class FPlayListEditTool;
class SPatternInstance;
class SPlayListPanel;
struct FPatternInstanceClickData;

struct FPatternInstanceSelectData
{
	FPatternInstance* PatternInstance = nullptr;
	AudioFrame MidiTimestampOffset;

	operator bool() const
	{
		return PatternInstance != nullptr;
	}
};

enum class EPlayListToolType
{
	Select,
	Write,
	DragView,
	Count
};

class FPlayListEdit
{
public:
	FPlayListEdit(TSharedRef<SPlayListPanel> InPlayListPanel);
	
	FReply OnMouseButtonDown(const FGeometry& MyGeometry, const FPointerEvent& MouseEvent);
	FReply OnMouseButtonUp(const FGeometry& MyGeometry, const FPointerEvent& MouseEvent);
	FReply OnMouseMove(const FGeometry& MyGeometry, const FPointerEvent& MouseEvent);
	FReply OnKeyDown(const FGeometry& MyGeometry, const FKeyEvent& InKeyEvent);
	FReply OnKeyUp(const FGeometry& MyGeometry, const FKeyEvent& InKeyEvent);
	int32 OnPaint(const FPaintArgs& Args, const FGeometry& AllottedGeometry, const FSlateRect& MyCullingRect, FSlateWindowElementList& OutDrawElements, int32 LayerId, const FWidgetStyle& InWidgetStyle, bool bParentEnabled) const;

	FPatternInstanceSelectData GetPatternInstanceAt(const FVector2f& LocalPosition) const;
	FPatternInstanceSelectData GetPatternUnderCursor() const;
	FReply OnPatternInstancePressed(FPatternInstanceClickData D);
	FReply OnPatternInstanceReleased(FPatternInstanceClickData D);

	TSharedRef<SPatternInstance> CreatePatternInstanceWidget(FPatternInstance* PatternInstance);

	bool IsMultiSelected() const { return Selector.Num() > 1; }
	FVector2f GetMouseDownPos() const { return MouseDownPos; }
	AudioFrame GetSnapFrame() const;
	float GetMidiTickToPixelScaler() const;
	float GetSampleToPixelScaler() const;
	AudioFrame GetFrameUnderCursor() const;
	AudioFrame PixelToFrame(float Pixel) const;
	int32 GetTrackIndexUnderCursor() const;
	int32 GetTrackIndexByPos(float Height) const;

	
	void SetTempTool(EPlayListToolType InToolType);
	void ClearTempTool();
	
	FPlayListSelector Selector;
	TWeakPtr<SPlayListPanel> PlayListPanel;
protected:
	void OnMouseEndDrag(FVector2f InMouseEndPos);
	bool OnMouseDrag = false;
private:
	FPlayListEditTool* TryEnableTempTool();

	bool IsPatternInstancePressed = false;
	bool HasSequenceModified = false;
	FVector2f MouseDownPos;

	TArray<FKey> PressedKeys;

	FPlayListEditTool* GetCurrentTool() const { return TempTool ? TempTool : CurrentEditTool; }
	FPlayListEditTool* TempTool = nullptr;
	FPlayListEditTool* CurrentEditTool = nullptr;
	TMap<EPlayListToolType, FPlayListEditTool*> EditToolMap;
	TMap<EPlayListToolType, TArray<FKey>> ToolKeyMap;
};
