﻿#include "PlayListSelector.h"

#include "Pattern/PatternInstance.h"

FPlayListSelector::FPlayListSelector()
{
	FMidiSequencer::Get().OnDeletePatternInstance.AddRaw(this, &FPlayListSelector::OnDeletePatternInstance);
}

FPlayListSelector::~FPlayListSelector()
{
	FMidiSequencer::Get().OnDeletePatternInstance.RemoveAll(this);
}

void FPlayListSelector::AddPatternInstance(FPatternInstance* PatternInstance)
{
	if (PatternInstanceList.Contains(PatternInstance))
		return;
	FData Data;
	Data.PatternInstance = PatternInstance;
	Data.OriginalTrackIndex = PatternInstance->TrackIndex;
	Data.OriginalMidiTimestamp = PatternInstance->Pos;
	
	PatternInstanceList.Add(Data);
	OnInstanceSelected.Broadcast(PatternInstance);
}

void FPlayListSelector::UnSelect(FPatternInstance* PatternInstance)
{
	if (PatternInstanceList.RemoveAll([PatternInstance](const FData& Data) { return Data.PatternInstance == PatternInstance; }))
	{
		OnInstanceDeselected.Broadcast(PatternInstance);
	}
}

void FPlayListSelector::UnSelect()
{
	for (const auto& D : PatternInstanceList)
	{
		OnInstanceDeselected.Broadcast(D.PatternInstance);
	}
	PatternInstanceList.Reset();
}

void FPlayListSelector::ResetAnchorPoint()
{
	for (auto& D : PatternInstanceList)
	{
		D.OriginalTrackIndex = D.PatternInstance->TrackIndex;
		D.OriginalMidiTimestamp = D.PatternInstance->Pos;
	}
}

void FPlayListSelector::EndSetting()
{
	for (auto& D : PatternInstanceList)
	{
		D.PatternInstance->RequestEndSetting();
	}
}

void FPlayListSelector::SetTrackIndex(int32 TrackIndex)
{
	const int32 Delta = TrackIndex - BaseTrackIndex;
	for (auto& D : PatternInstanceList)
	{
		D.PatternInstance->TrackIndex = D.OriginalTrackIndex + Delta;
	}
}

bool FPlayListSelector::SetFrameTimestamp(AudioFrame MidiTimestamp)
{
	double TimestampOffset = BaseMidiTimestamp - MidiTimestamp + MidiTimestampDelta; // 点击位置的MidiTick
	TimestampOffset = FMath::GridSnap<AudioFrame>(TimestampOffset, SnapMidiTimestamp);
	
	for (const FData& D : PatternInstanceList)
	{
		const double NewTimestamp = D.OriginalMidiTimestamp - TimestampOffset;
		D.PatternInstance->Pos = FMath::Max(0, NewTimestamp);
	}

	if (LastMidiTimestampOffset != TimestampOffset)
	{
		LastMidiTimestampOffset = TimestampOffset;
		return true;
	}
	return false;
}

bool FPlayListSelector::IsSelected(FPatternInstance* PatternInstance) const
{
	return PatternInstanceList.Contains(PatternInstance);
}

void FPlayListSelector::OnDeletePatternInstance(FPatternInstance* PatternInstance)
{
	PatternInstanceList.RemoveAll([PatternInstance](const FData& Data) { return Data.PatternInstance == PatternInstance; });
}
