﻿#pragma once
#include "Attribute.h"
#include "Singleton/MidiSequencer.h"

class FPatternInstance;

class FPlayListSelector
{
	struct FData
	{
		FPatternInstance* PatternInstance;
		int OriginalTrackIndex;
		double OriginalMidiTimestamp;

		bool operator==(FPatternInstance* In) const
		{
			return PatternInstance == In;
		}

		bool operator==(const FData& In) const
		{
			return PatternInstance == In.PatternInstance;
		}
	};
public:
	FPlayListSelector();
	~FPlayListSelector();

	void AddPatternInstance(FPatternInstance* PatternInstance);
	void UnSelect(FPatternInstance* PatternInstance);
	void UnSelect();
	
	void BeginDrag(int32 InBaseTrackIndex, AudioFrame InSnapFrame, AudioFrame InMidiTimestampDelta, AudioFrame InBaseMidiTimestamp)
	{
		BaseTrackIndex = InBaseTrackIndex;
		SnapMidiTimestamp = InSnapFrame;
		MidiTimestampDelta = InMidiTimestampDelta;
		BaseMidiTimestamp = InBaseMidiTimestamp;
		LastMidiTimestampOffset = 0;
	}

	int32 Num() const { return PatternInstanceList.Num(); }

	void ResetAnchorPoint();
	void EndSetting();

	void SetTrackIndex(int32 TrackIndex);
	bool SetFrameTimestamp(AudioFrame MidiTimestamp);

	bool IsSelected(FPatternInstance* PatternInstance) const;
	
	FPatternInstanceDelegate OnInstanceSelected;
	FPatternInstanceDelegate OnInstanceDeselected;
private:
	void OnDeletePatternInstance(FPatternInstance* PatternInstance);

	TArray<FData> PatternInstanceList;
	int32 BaseTrackIndex = 0;
	AudioFrame MidiTimestampDelta = 0;
	AudioFrame BaseMidiTimestamp = 0;
	AudioFrame SnapMidiTimestamp = 0;
	double LastMidiTimestampOffset = 0;
};
