﻿// Fill out your copyright notice in the Description page of Project Settings.

#pragma once

#include "CoreMinimal.h"
#include "FTrackHeightManager.h"
#include "PlayListSelector.h"
#include "SCompoundWidget.h"
#include "Pattern/PatternInstance.h"
#include "UI/Widget/IChildWindow.h"
#include "UI/Widget/Pattern/SPatternInstance.h"

class SPlayListTimeLine;
class FPlayListEdit;
class SConstraintCanvas;
class SVerticalBox;
class SScrollBox;

/**
 * 
 */
class ARONA_API SPlayListPanel : public SCompoundWidget, public IChildWindow
{
public:
	SLATE_BEGIN_ARGS(SPlayListPanel)
		{}

	SLATE_ATTRIBUTE(float, TrackHeight)
	SLATE_ATTRIBUTE(TRange<AudioFrame>, FrameRange)
	
	SLATE_END_ARGS()

	/** Constructs this widget with InArgs */
	void Construct(const FArguments& InArgs);
	virtual int32 OnPaint(const FPaintArgs& Args, const FGeometry& AllottedGeometry, const FSlateRect& MyCullingRect, FSlateWindowElementList& OutDrawElements, int32 LayerId, const FWidgetStyle& InWidgetStyle, bool bParentEnabled) const override;
	virtual FVector2D ComputeDesiredSize(float LayoutScaleMultiplier) const override;
	virtual FReply OnMouseButtonDown(const FGeometry& MyGeometry, const FPointerEvent& MouseEvent) override;
	virtual FReply OnMouseButtonUp(const FGeometry& MyGeometry, const FPointerEvent& MouseEvent) override;
	virtual FReply OnMouseMove(const FGeometry& MyGeometry, const FPointerEvent& MouseEvent) override;
	virtual FReply OnMouseWheel(const FGeometry& MyGeometry, const FPointerEvent& InMouseEvent) override;
	virtual FReply OnKeyDown(const FGeometry& MyGeometry, const FKeyEvent& InKeyEvent) override;
	virtual FReply OnKeyUp(const FGeometry& MyGeometry, const FKeyEvent& InKeyEvent) override;
	
	// double PixelToMidiTick(float Pixel) const { return Pixel / GetMidiTickToPixelScaler() + MidiTickRange.Get().GetLowerBoundValue(); }
	AudioFrame PixelToFrame(float Pixel) const { return Pixel / GetSampleToPixelScaler() + ViewRange.Get().GetLowerBoundValue(); }
	float MidiTickToPixel(MidiTick InMidiTick) const;
	float FrameToPixel(AudioFrame InFrame) const;
	float GetMidiTickToPixelScaler() const { return 0.125f; }
	AudioFrame GetSampleToPixelScaler() const;
	AudioFrame GetPixelToSampleScaler() const;
	TAttribute<TRange<AudioFrame>> GetViewRange() const { return ViewRange; }

	virtual TSharedRef<SWidget> GetWidget() override { return AsShared(); }
	void UpdateAllPatternInstance();

	FTrackHeightManager TrackHeightManager;

	int32 TrackCount = 128;
	float SpacingLineThickness = 1.f;
	FLinearColor SpacingLineColor = FLinearColor::Gray;
private:
	FMargin CalculatePatternInstancePos(FPatternInstance* PatternInstance) const;
	void OnCreatePatternInstance(FPatternInstance* PatternInstance);
	void OnDeletePatternInstance(FPatternInstance* PatternInstance);
	
	TSharedPtr<SConstraintCanvas> TrackCanvas;
	TSharedPtr<FPlayListEdit> PlayListEdit;

	TAttribute<TRange<AudioFrame>> ViewRange;
};
