﻿#pragma once
#include "Singleton.h"
#include "Thread/ThreadMessage.h"

class FChannelInterface;
struct FMixerTrackLink;
class FMixerTrack;
class FDummyMixerTrack;
class FInstrumentMixerTrack;
class FPluginHost;

DECLARE_MULTICAST_DELEGATE_OneParam(FOnMixerTrackEvent, FMixerTrack*);
DECLARE_MULTICAST_DELEGATE_TwoParams(FOnMixerTrackEffectEvent, FMixerTrack*, FPluginHost*);

class FMixerList : public TSingleton<FMixerList>, public TArray<FMixerTrack*>
{
public:
	virtual void Init() override;
	virtual void Release() override;

	virtual FName GetName() override { return "MixerList"; }

	FORCEINLINE FMixerTrack* GetMaster() const { return GetData()[0]; }
	FDummyMixerTrack* CreateDummyTrack(const FString& Name);
	FInstrumentMixerTrack* CreateInstrumentTrack(FPluginHost* Host);
	void RemoveTrack(FMixerTrack* Track);
	void RemoveTrack(FPluginHost* Instrument);
	void RemoveTrack_AudioThread(FMixerTrack* InTrack);

	void Process(int32 NumSamples);

	void ResetAllTracks();

	void BuildProcessNode();
	void RequestBuildProcessNode();
	
	FOnMixerTrackEvent OnMixerTrackCreated;
	FOnMixerTrackEvent OnMixerTrackRemoved;
	FOnMixerTrackEffectEvent OnMixerTrackEffectAdded;
	FOnMixerTrackEffectEvent OnMixerTrackEffectRemoved;

	FDummyMixerTrack* ZeroTrack = nullptr; // 用于没有任何音频输出的情况
private:
	static void ProcessInternal(FMixerTrack* Track, int32 NumSamples, TArray<FMixerTrack*>& ProcessedTracks);
	int32 BuildProcessNodeInternal(FMixerTrack* Track, TMap<FMixerTrack*, int32>& ProcessedTracks, int32 Layer);
	void BuildInstrumentProcessNode(FPluginHost* PluginHost, TMap<FMixerTrack*, int32>& ProcessedTracks);
	void BuildEffectChannelInterface(FMixerTrack* Track, FChannelInterface* ChannelInterface, TMap<FMixerTrack*, int32>& ProcessedTracks);

	TMap<int32, TArray<FMixerTrack*>> LayerTracks;
};
